//---------------------------------------------------------------------
//  Copyright (C) Microsoft Corporation.  All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
//---------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace DesignSurfaceExtenderLibrary.Extensions
{
    internal class MarginBehavior : BoundsBehavior
    {
        internal MarginBehavior( IServiceProvider serviceProvider )
            : base( serviceProvider )
        {
            this.propertyName = "Margin";
            this.transactionName = "Margin drag";
        }

        protected override void SetBoundsProperty( PropertyDescriptor pd )
        {
            if( pd != null )
            {
                Rectangle translatedBounds = new Rectangle(
                    this.behaviorServiceValue.ControlToAdornerWindow( this.relatedControl ),
                    this.relatedControl.Size );

                switch( this.selectedEdge )
                {
                    case EdgeSelection.Left:
                    {
                        int leftMargin = translatedBounds.Left - this.currentPosition.X;

                        if( leftMargin >= 0 )
                        {
                            Padding margin = new Padding(
                                leftMargin,
                                this.relatedControl.Margin.Top,
                                this.relatedControl.Margin.Right,
                                this.relatedControl.Margin.Bottom );

                            pd.SetValue( this.relatedControl, margin );
                        }

                        break;
                    }

                    case EdgeSelection.Top:
                    {
                        int topMargin = translatedBounds.Top - this.currentPosition.Y;

                        if( topMargin >= 0 )
                        {
                            Padding margin = new Padding(
                                this.relatedControl.Margin.Left,
                                topMargin,
                                this.relatedControl.Margin.Right,
                                this.relatedControl.Margin.Bottom );

                            pd.SetValue( this.relatedControl, margin );
                        }

                        break;
                    }

                    case EdgeSelection.Right:
                    {
                        int rightMargin = this.currentPosition.X - translatedBounds.Right;

                        if( rightMargin >= 0 )
                        {
                            Padding margin = new Padding(
                                this.relatedControl.Margin.Left,
                                this.relatedControl.Margin.Top,
                                rightMargin,
                                this.relatedControl.Margin.Bottom );

                            pd.SetValue( this.relatedControl, margin );
                        }

                        break;
                    }

                    case EdgeSelection.Bottom:
                    {
                        int bottomMargin = this.currentPosition.Y - translatedBounds.Bottom;

                        if( bottomMargin >= 0 )
                        {
                            Padding margin = new Padding(
                                this.relatedControl.Margin.Left,
                                this.relatedControl.Margin.Top,
                                this.relatedControl.Margin.Right,
                                bottomMargin );

                            pd.SetValue( this.relatedControl, margin );
                        }

                        break;
                    }
                
                    case EdgeSelection.None:
                    default:
                    {
                        break;
                    }
                }
            }
        }
    }
}
